﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using Styx;
using Styx.WoWInternals;

namespace HighVoltz
{
    public partial class MainForm : Form
    {
        Radar _radar = Radar.Instance;
        RadarSettings _settings = RadarSettings.Instance;
        public MainForm()
        {
            InitializeComponent();
        }

        private bool _keyBindMode = false;
        private void KeyBindButton_Click(object sender, EventArgs e)
        {
            if (!_keyBindMode)
            {
                Button btn = (Button)sender;
                btn.BackColor = System.Drawing.SystemColors.GradientInactiveCaption;
                btn.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
                btn.Text = "Press key combination";
                _keyBindMode = true;
            }
        }

        private void KeyBindButton_KeyDown(object sender, KeyEventArgs e)
        {
            // return if not changing keybinds.
            if (_keyBindMode)
            {
                Button btn = (Button)sender;
                var key = GetKeyBind((Button)sender, e);
                if (key != Keys.None)
                {
                    switch (btn.Name)
                    {
                        case "RadarKeyBindButton":
                            _settings.RadarKeybind = key;
                            _radar.RadarKeyBind.KeyBind = key;
                            break;
                    }
                    _keyBindMode = false;
                }
            }
        }


        Keys GetKeyBind(Button sender, KeyEventArgs e)
        {
            // convert modifier keys to string to be displayed on button
            string keybindstring = (e.Shift ? "Shift " : "") + (e.Control ? "Ctrl " : "") + (e.Alt ? "Alt " : "");
            sender.Text = keybindstring;
            if (e.KeyCode == Keys.ShiftKey || e.KeyCode == Keys.ControlKey || e.KeyCode == Keys.Menu)
            {
                return Keys.None;
            }
            // display numbers in button text correctly.
            if (e.KeyCode >= Keys.D0 && e.KeyCode <= Keys.D9)
            {
                keybindstring += e.KeyValue - 48;
            }
            else
            {
                keybindstring += e.KeyCode.ToString();
            }
            sender.Text = keybindstring;
            sender.FlatStyle = System.Windows.Forms.FlatStyle.Standard;
            sender.BackColor = System.Drawing.SystemColors.Control;
            return e.KeyData;
        }

        private void MainForm_Shown(object sender, EventArgs e)
        {
            TrackPlayersCheck.Checked = _settings.TrackPlayers;
            TrackHostilePlayersCheck.Checked = _settings.HostilePlayersOnly;
            TrackHostilePlayersCheck.Enabled = TrackPlayersCheck.Checked;

            TrackNPCsCheck.Checked = _settings.TrackNPCs;
            TrackHostileNPCsCheck.Checked = _settings.HostileNPCsOnly;
            TrackHostileNPCsCheck.Enabled = TrackNPCsCheck.Checked;

            TrackObjectsCheck.Checked = _settings.TrackObjects;


            TrackOnyxCheck.Checked = _settings.TrackOnyx;
            TrackLostTreasuresCheck.Checked = _settings.TrackLostTreasures;
            

            CustomTrackText.Text = _settings.CustomName;

            RadarWidthText.Text = _settings.RadarSize.ToString();

            if (_settings.RadarKeybind == 0)
                RadarKeyBindButton.Text = "Click to set Keybind";
            else
                RadarKeyBindButton.Text = VkeyToString(_settings.RadarKeybind);
        }

        string VkeyToString(System.Windows.Forms.Keys vKey)
        {
            // extract modifier keys                 
            string keybindstring = ((vKey & Keys.Shift) != 0 ? "Shift " : "") + ((vKey & Keys.Control) != 0 ? "Ctrl " : "") + ((vKey & Keys.Alt) != 0 ? "Alt " : "");
            // Shift = 0x00010000, Ctrl= 0x00020000, Alt = 0x00040000
            // remove modifer bits
            vKey ^= (Keys.Shift & vKey) | (Keys.Control & vKey) | (Keys.Alt & vKey);
            // display numbers in button text correctly.
            switch (vKey)
            {
                case Keys.D0:
                    keybindstring += "0";
                    break;
                case Keys.D1:
                    keybindstring += "1";
                    break;
                case Keys.D2:
                    keybindstring += "2";
                    break;
                case Keys.D3:
                    keybindstring += "3";
                    break;
                case Keys.D4:
                    keybindstring += "4";
                    break;
                case Keys.D5:
                    keybindstring += "5";
                    break;
                case Keys.D6:
                    keybindstring += "6";
                    break;
                case Keys.D7:
                    keybindstring += "7";
                    break;
                case Keys.D8:
                    keybindstring += "8";
                    break;
                case Keys.D9:
                    keybindstring += "9";
                    break;
                default:
                    keybindstring += vKey.ToString();
                    break;
            }
            return keybindstring;
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            _settings.Save();
        }

        private void TrackPlayersCheck_CheckedChanged(object sender, EventArgs e)
        {
            _settings.TrackPlayers = TrackPlayersCheck.Checked;
            TrackHostilePlayersCheck.Enabled = _settings.TrackPlayers;
        }

        private void TrackHostilePlayersCheck_CheckedChanged(object sender, EventArgs e)
        {
            _settings.HostilePlayersOnly = TrackHostilePlayersCheck.Checked;
        }

        private void TrackNPCsCheck_CheckedChanged(object sender, EventArgs e)
        {
            _settings.TrackNPCs = TrackNPCsCheck.Checked;
            TrackHostileNPCsCheck.Enabled = _settings.TrackNPCs;
        }

        private void TrackHostileNPCsCheck_CheckedChanged(object sender, EventArgs e)
        {
            _settings.HostileNPCsOnly = TrackHostileNPCsCheck.Checked;
        }

        private void TrackObjectsCheck_CheckedChanged(object sender, EventArgs e)
        {
            _settings.TrackObjects = TrackObjectsCheck.Checked;
        }

        private void RadarWidthText_TextChanged(object sender, EventArgs e)
        {
            double size;
            double.TryParse(RadarWidthText.Text,NumberStyles.AllowDecimalPoint,
                CultureInfo.InvariantCulture,out size);
            _settings.RadarSize = size;
            if (RadarWindow.IsValid)
            {
                var windowInfo = Win32.GetWindowInfo(StyxWoW.Memory.Process.MainWindowHandle);
                RadarWindow.Resize(windowInfo.rcClient);
            }
        }

        private void CustomTrackText_TextChanged(object sender, EventArgs e)
        {
            int id;
            int.TryParse(CustomTrackText.Text, out id);
            _settings.CustomId = id;
            _settings.CustomName = CustomTrackText.Text;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }

        private void TrackOnyxCheck_CheckedChanged(object sender, EventArgs e)
        {
            _settings.TrackOnyx = TrackOnyxCheck.Checked;
        }

        private void TrackLostTreasuresCheck_CheckedChanged(object sender, EventArgs e)
        {
            _settings.TrackLostTreasures = TrackLostTreasuresCheck.Checked;
        }
    }
}
