﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Data;
using System.Threading;
using Styx;
using Styx.WoWInternals;
using System.Runtime.InteropServices;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Controls;
using Styx.WoWInternals.WoWObjects;
using Styx.Helpers;


namespace HighVoltz
{
    public class RadarWindow : Window
    {
        
        #region Statics
          
        static public RadarWindow Instance { get; private set; }
        static public void Close2()
       

        {
            if (IsValid)
            {
                if (Thread.CurrentThread != Instance.Dispatcher.Thread)
                    Instance.Dispatcher.BeginInvoke((Action)(() => { Instance.Close(); }));
                else
                    Instance.Close();
            }
        }

        static public bool IsValid { get; private set; }


        static public bool IsInForeground()
        {
            if (IsValid)
            {
                if (Thread.CurrentThread != Instance.Dispatcher.Thread)
                    return (bool)Instance.Dispatcher.Invoke((Func<bool>)(() => Win32.GetForegroundWindow() == Instance.HWind));
                else
                    return Win32.GetForegroundWindow() == Instance.HWind;
            }
            return false;
        }

        static public Point GetPos(Rect rect)
        {
            var screenBounds = System.Windows.Forms.Screen.PrimaryScreen.Bounds;
            Point point = new Point();
            double scaleX = 1 / ((rect.right - rect.left) / (double)screenBounds.Width);
            double scaleY = 1 / ((rect.bottom - rect.top) / (double)screenBounds.Height);
            if (IsValid)
            {
                if (Thread.CurrentThread != Instance.Dispatcher.Thread)
                    Instance.Dispatcher.Invoke((Action)(() =>
                    {
                        point.X = ((Instance.Left - rect.left) + (Instance.Width / 2)) * scaleX;
                        point.Y = ((Instance.Top - rect.top) + (Instance.Width / 2)) * scaleY;
                    }));
                else
                {
                    point.X = (rect.left + (RadarSettings.Instance.RadarSize / 2)) * (1 / scaleX);
                    point.X = (rect.top + (RadarSettings.Instance.RadarSize / 2)) * (1 / scaleY);
                }
            }
            return point;
        }

        static public void HideWindow(bool hide)
        {
            if (IsValid)
            {
                if (Thread.CurrentThread != Instance.Dispatcher.Thread)
                    Instance.Dispatcher.BeginInvoke((Action<bool>)((t) => { _mainGrid.Visibility = hide ? Visibility.Collapsed : Visibility.Visible; }), hide);
                else
                    _mainGrid.Visibility = hide ? Visibility.Collapsed : Visibility.Visible;
            }
        }

        static public void SetTop(bool top)
        {
            if (IsValid)
            {
                if (Thread.CurrentThread != Instance.Dispatcher.Thread)
                    Instance.Dispatcher.BeginInvoke((Action<bool>)((t) => { Instance.Topmost = t; }), top);
                else
                    Instance.Topmost = top;
            }
        }

        static public void Resize(Rect rect)
        {
            if (IsValid)
            {
                if (Thread.CurrentThread != Instance.Dispatcher.Thread)
                {
                    Instance.Dispatcher.BeginInvoke((Action)(() => { DoResize(rect); }));
                }
                else
                    DoResize(rect);
            }
        }

        static void DoResize(Rect rect)
        {
            var screenBounds = System.Windows.Forms.Screen.PrimaryScreen.Bounds;
            double scaleX = (rect.right - rect.left) / (double)screenBounds.Width;
            double scaleY = (rect.bottom - rect.top) / (double)screenBounds.Height;
            Instance.Width = _settings.RadarSize * scaleX;
            Instance.Height = Instance.Width;
            Instance.Left = rect.left + (((_settings.RadarX) - (_settings.RadarSize / 2)) * scaleX);
            Instance.Top = rect.top + (((_settings.RadarY) - (_settings.RadarSize / 2)) * scaleY);
        }

        static public void DragMove(int x, int y, Rect bounds)
        {
            if (IsValid)
            {
                if (Thread.CurrentThread != Instance.Dispatcher.Thread)
                {
                    Instance.Dispatcher.BeginInvoke((Action)(() => { DoDragMove(x, y, bounds); }));
                }
                else
                    DoDragMove(x, y, bounds);
            }
        }
        static void DoDragMove(int x, int y, Rect bounds)
        {
            double newX = x - _mouseXOffset;
            double newY = y - _mouseYOffset;
            if (newX < bounds.left)
                newX = bounds.left;
            else if (newX + Instance.Width > bounds.right)
                newX = bounds.right - Instance.Width;
            if (newY < bounds.top)
                newY = bounds.top;
            else if (newY + Instance.Height > bounds.bottom)
                newY = bounds.bottom - Instance.Height;
            Instance.Left = newX;
            Instance.Top = newY;
        }

        public static void UpdateRadar()
        {
         
            
            try
            {
                using (var d = _radarCanvas.Dispatcher.DisableProcessing())
                {   // check if canvas has objects and creat them if not.
                    lock (Radar.Locker)
                    {
                        foreach (var obj in Radar.Objects)
                        {
                            bool hasObj = false;
                            for (int i = _radarCanvas.Children.Count - 1; i >= 0; i--)
                            {
                                IWoWObjectShape ishape = (IWoWObjectShape)_radarCanvas.Children[i];
                                if (ishape.ObjGuid == obj.Key)
                                {
                                    hasObj = true;
                                    break;
                                }
                            }
                            // lets add obj to list
                            if (!hasObj)
                            {
                                try
                                { // safty net if object is invalid. fail silent if so..

                                    uint[] LostObjects = {

                                        213741,
                                        213364,
                                        214439,
                                        213972,
                                        213967,
                                        213970,
                                        213969,
                                        213742,
                                        213768,
                                        213966,
                                        213964,
                                        213653,
                                        213749,
                                        213842,
                                        213968,
                                        213971,
                                        213765,
                                        213962,
                                        213363,
                                        213960,
                                        213649,
                                        213651,
                                        213751,
                                        64272,
                                        65552,
                                        64004,
                                        64191

                                    };

                                    if (obj.Value.GetType() == typeof(WoWUnit))
                                        _radarCanvas.Children.Add(new NpcShape(obj.Value));
                                    else if (obj.Value.GetType() == typeof(WoWPlayer))
                                        _radarCanvas.Children.Add(new PlayerShape(obj.Value));
                                    else if (obj.Value.GetType() == typeof(LocalPlayer))
                                        _radarCanvas.Children.Add(new LocalPlayerShape(obj.Value));
                                    else if (obj.Value.Entry == 214945)
                                        _radarCanvas.Children.Add(new WoWEggShape(obj.Value));
                                    else if (LostObjects.Contains(obj.Value.Entry))
                                        _radarCanvas.Children.Add(new WoWTreasureShape(obj.Value));
                                    else if (obj.Value.GetType() == typeof(WoWGameObject))
                                        _radarCanvas.Children.Add(new WoWObjectShape(obj.Value));
                                    
                                }
                                catch { }
                            }
                        }
                        // remove objects that are nolonger in objectManager
                        for (int i = _radarCanvas.Children.Count - 1; i >= 0; i--)
                        {
                            IWoWObjectShape ishape = (IWoWObjectShape)_radarCanvas.Children[i];
                            if (!Radar.Objects.ContainsKey(ishape.ObjGuid))
                                _radarCanvas.Children.RemoveAt(i);
                            else
                            {
                                WoWObject obj = Radar.Objects[ishape.ObjGuid];
                                try
                                {
                                    if (obj.IsValid)
                                    {
                                        ishape.Update();
                                        double scaleX = (RadarImage.ActualWidth / 300d) * RadarScaleTrans.ScaleX;
                                        double scaleY = (RadarImage.ActualHeight / 300d) * RadarScaleTrans.ScaleY;
                                        double centerX = RadarImage.ActualWidth / 2;
                                        double centerY = RadarImage.ActualHeight / 2;
                                        Canvas.SetLeft((Shape)ishape, ((Me.Y - obj.Location.Y) * scaleX) + centerX);
                                        Canvas.SetTop((Shape)ishape, ((Me.X - obj.Location.X) * scaleY) + centerY);
                                    }
                                    else
                                        _radarCanvas.Children.RemoveAt(i);
                                }
                                catch
                                {
                                    _radarCanvas.Children.RemoveAt(i);
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                _radar.Log(ex.ToString());
            }
        }

        #endregion
        static WoWPlayer Me { get { return StyxWoW.Me; } }
        static Radar _radar = Radar.Instance;
        static RadarSettings _settings = RadarSettings.Instance;
        static Grid _mainGrid;
        static Image RadarImage;
        static ScaleTransform RadarScaleTrans;
        static Canvas _radarCanvas;

        public IntPtr HWind { get; private set; }
        public RadarWindow()
        {
            try
            {
                Instance = this;
                Initialize();
            }
            catch (Exception ex)
            {
                _radar.Log(ex.ToString());
            }
        }

        void Initialize()
        {
            Loaded += RadarWindow_Loaded;
            IsDragged = false;
            AllowsTransparency = true;
            ShowInTaskbar = false;
            ResizeMode = System.Windows.ResizeMode.NoResize;
            WindowStyle = System.Windows.WindowStyle.None;
            Background = new SolidColorBrush(Colors.Transparent);
            Topmost = true;
            var winInfo = Win32.GetWindowInfo(StyxWoW.Memory.Process.MainWindowHandle);
            DoResize(winInfo.rcClient);

            Binding HeightBinding = new Binding();
            HeightBinding.Source = this;
            HeightBinding.Path = new PropertyPath("Width");
            SetBinding(HeightProperty, HeightBinding);
            _mainGrid = new Grid() { Background = new SolidColorBrush(Colors.Transparent) };
            _mainGrid.VerticalAlignment = VerticalAlignment.Center;
            _mainGrid.HorizontalAlignment = HorizontalAlignment.Center;
            Content = _mainGrid;

            DrawingGroup dg = new DrawingGroup();

            GeometryDrawing circle100y = new GeometryDrawing();
            circle100y.Pen = new Pen(new SolidColorBrush(Color.FromScRgb(0.8f, 0f, 0f, 0f)), 1d);
            circle100y.Pen.DashStyle = new DashStyle(new DoubleCollection { 4f, 8f }, 0);
            circle100y.Brush = new SolidColorBrush(Colors.Transparent);
            EllipseGeometry c1 = new EllipseGeometry();
            c1.RadiusX = 200;
            c1.RadiusY = 200;
            circle100y.Geometry = c1;
            dg.Children.Add(circle100y);

            GeometryDrawing circle50y = new GeometryDrawing();
            circle50y.Pen = new Pen(new SolidColorBrush(Color.FromScRgb(0.8f, 0f, 0f, 0f)), 1d);
            circle50y.Pen.DashStyle = new DashStyle(new DoubleCollection { 4f, 8f }, 0);
            circle50y.Brush = new SolidColorBrush(Colors.Transparent);
            EllipseGeometry c2 = new EllipseGeometry();
            c2.RadiusX = 100;
            c2.RadiusY = 100;
            circle50y.Geometry = c2;
            dg.Children.Add(circle50y);

            GeometryDrawing radarCircle = new GeometryDrawing();
            radarCircle.Pen = new Pen(new SolidColorBrush(Colors.DarkRed), 1d);
            radarCircle.Brush = new SolidColorBrush(Color.FromScRgb(0.12f, 1f, 0f, 0f));
            EllipseGeometry c3 = new EllipseGeometry();
            c3.RadiusX = 240;
            c3.RadiusY = 240;
            radarCircle.Geometry = c3;
            dg.Children.Add(radarCircle);

            DrawingImage di = new DrawingImage(dg);
            RadarScaleTrans = new ScaleTransform();
            RadarImage = new Image();
            RadarImage.Source = di;
            RadarImage.RenderTransform = RadarScaleTrans;

            _mainGrid.Children.Add(RadarImage);
            _radarCanvas = new Canvas();
            _mainGrid.Children.Add(_radarCanvas);
        }

        void RadarWindow_Loaded(object sender, RoutedEventArgs e)
        {
            Loaded -= RadarWindow_Loaded;
            HWind = new WindowInteropHelper(this).Handle;
            IsValid = true;
        }

        public bool IsDragged { get; set; }
        static double _mouseXOffset;
        static double _mouseYOffset;

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            IsDragged = true;
            _mouseXOffset = e.GetPosition(this).X;
            _mouseYOffset = e.GetPosition(this).Y;
        }

        protected override void OnMouseWheel(MouseWheelEventArgs e)
        {
            double zoom = e.Delta > 0 ? .05 : -.05;
            if ((RadarScaleTrans.ScaleX <= 1 && zoom < 0) || (RadarScaleTrans.ScaleX >= 4 && zoom > 0))
                return;
            RadarScaleTrans.CenterX = RadarImage.ActualWidth / 2;
            RadarScaleTrans.CenterY = RadarImage.ActualHeight / 2;
            //RadarScaleTrans.CenterX = e.GetPosition(RadarImage).X;
            //RadarScaleTrans.CenterY = e.GetPosition(RadarImage).Y;
            RadarScaleTrans.ScaleX += zoom;
            RadarScaleTrans.ScaleY += zoom;
        }
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            _radarCanvas.Children.Clear();
            base.OnClosing(e);
            DialogResult = true;
            _radar.Log("CLosing");
            IsValid = false;
        }
    }
}
