﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Shapes;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System.Windows.Media;
using Styx.Helpers;
using System.Windows.Controls;
using Styx;
using HighVoltz.ContextMenuCommands;
using System.Collections;
using System.Windows.Input;
using System.Windows;

namespace HighVoltz
{
    public interface IWoWObjectShape
    {
        void Update();
        ulong ObjGuid { get; }
    }

    public class ReactionColors
    {
        public static Color FriendlyColor = Colors.Green;
        public static Color FriendlyPenColor = Colors.DarkGreen;
        public static Color NeutralColor = Colors.Yellow;
        public static Color NeutralPenColor = Colors.YellowGreen;
        public static Color HostileColor = Colors.Red;
        public static Color HostilePenColor = Colors.DarkRed;
        public static Color PlayerColor = Colors.Blue;
        public static Color PlayerPenColor = Colors.DarkBlue;
        public static Color ObjectColor = Colors.Yellow;
        public static Color ObjectPenColor = Colors.Black;
        public static Color EggColor = Colors.YellowGreen;
        public static Color TreasureColor = Colors.Aqua;
    }

    public class ObjectShape : Shape, IWoWObjectShape
    {
        protected WoWObject unit;
        protected RotateTransform rotXform;
        public ulong ObjGuid { get; private set; }
        virtual protected MenuItem[] menuItems { get; set; }

        public ObjectShape(WoWObject unit)
        {
            this.unit = unit;
            ObjGuid = unit.Guid;
            rotXform = new RotateTransform(0d, 0d, 0d);
            RenderTransform = rotXform;
            ToolTipService.SetToolTip(this, string.Format("{0} Id:{1}", unit.Name, unit.Entry));
            ToolTipService.SetInitialShowDelay(this, 0);
            ToolTipService.SetShowDuration(this, 2000);
            ContextMenu = new ContextMenu();
            ContextMenu.Style = new Style();
            ContextMenu.ItemsSource = ContextMenuItems();
            Update();
        }

        protected override void OnContextMenuClosing(ContextMenuEventArgs e)
        {
            Win32.SetForegroundWindow(StyxWoW.Memory.Process.MainWindowHandle);
        }

        virtual public void Update() { }
        virtual protected MenuItem[] ContextMenuItems()
        {
            return new MenuItem[]{ new MenuItem(){
                Header="Copy ID", Command = new CopyIdCommand(unit), Style = new Style()},
            };
        }
        protected override Geometry DefiningGeometry { get { return PathGeometry.Parse("M -3,2 A 3,3 0 0 0 3,2 M -3,2 A 3,3 0 0 1 3,2 M 0,-3 L 0,-4 Z"); } }

    }

    public class NpcShape : ObjectShape
    {
        public NpcShape(WoWObject unit)
            : base(unit)
        {

            if (((WoWUnit)unit).MyReaction <= Styx.WoWUnitReaction.Unfriendly)
            {
                Fill = new SolidColorBrush(ReactionColors.HostileColor);
                Stroke = new SolidColorBrush(ReactionColors.HostilePenColor);
            }
            else if (((WoWUnit)unit).MyReaction >= Styx.WoWUnitReaction.Friendly)
            {
                Fill = new SolidColorBrush(ReactionColors.FriendlyColor);
                Stroke = new SolidColorBrush(ReactionColors.FriendlyPenColor);
            }
            else
            {
                Fill = new SolidColorBrush(ReactionColors.NeutralColor);
                Stroke = new SolidColorBrush(ReactionColors.HostilePenColor);
            }
        }
        protected float _lastAngle = float.MaxValue;
        override public void Update()
        {
            if (!unit.IsValid)
                throw new Exception();
            if (unit.Rotation != _lastAngle)
            {
                _lastAngle = unit.RotationDegrees;
                rotXform.Angle = 360 - _lastAngle;
            }
        }
        protected override Geometry DefiningGeometry
        {
            get
            {
                return PathGeometry.Parse("M -3,2 A 3,3 0 0 0 3,2 M -3,2 A 3,3 0 0 1 3,2 M 0,-3 L 0,-4 Z");
            }
        }
        protected override void OnMouseLeftButtonDown(System.Windows.Input.MouseButtonEventArgs e)
        {
            WoWUnit unit = ObjectManager.GetObjectByGuid<WoWUnit>(ObjGuid);
            if (unit != null && unit.IsValid)
                unit.Target();
        }
    }

    public class PlayerShape : NpcShape
    {
        public PlayerShape(WoWObject unit)
            : base(unit)
        {
            ToolTipService.SetToolTip(this, unit.Name);
        }
        protected override Geometry DefiningGeometry
        {
            get
            {
                return PathGeometry.Parse("M -0,-4 L 0,-3 L-3,3 L3,3 L0,-3 Z");
            }
        }
    }

    public class LocalPlayerShape : PlayerShape
    {
        public LocalPlayerShape(WoWObject unit)
            : base(unit)
        {
            Fill = new SolidColorBrush(ReactionColors.PlayerColor);
            Stroke = new SolidColorBrush(ReactionColors.PlayerPenColor);
        }
    }

    public class WoWObjectShape : ObjectShape
    {
        public WoWObjectShape(WoWObject unit)
            : base(unit)
        {
            Fill = new SolidColorBrush(ReactionColors.ObjectColor);
            Stroke = new SolidColorBrush(ReactionColors.ObjectPenColor);
        }

        protected override Geometry DefiningGeometry
        {
            get
            {
                return PathGeometry.Parse("M -0,-3 L-3,0 L0,3 L3,0 L0,-3 Z");
            }
        }
    }

    public class WoWEggShape : ObjectShape
    {
        public WoWEggShape(WoWObject unit)
            : base(unit)
        {
            Fill = new SolidColorBrush(ReactionColors.EggColor);
            Stroke = new SolidColorBrush(ReactionColors.ObjectPenColor);
        }

        protected override Geometry DefiningGeometry
        {
            get
            {
                return PathGeometry.Parse("M -3,2 A 3,3 0 0 0 3,2 M -3,2 A 3,3 0 0 1 3,2 M 0,-4 Z");

            }
        }
    }

    public class WoWTreasureShape : ObjectShape
    {
        public WoWTreasureShape(WoWObject unit)
            : base(unit)
        {
            Fill = new SolidColorBrush(ReactionColors.TreasureColor);
            Stroke = new SolidColorBrush(ReactionColors.ObjectPenColor);
        }

        protected override Geometry DefiningGeometry
        {
            get
            {
                return PathGeometry.Parse("M -3,2 A 3,3 0 0 0 3,2 M -3,2 A 3,3 0 0 1 3,2 M 0,-4 Z");

            }
        }
    }

}
